@@ -37,10 +37,10 @@ def soap_login(options):
 	if "--ssl" in options or "--ssl-secure" in options or "--ssl-insecure" in options:
 		if "--ssl-insecure" in options:
 			import ssl
-			from requests.packages.urllib3.exceptions import InsecureRequestWarning
+			import urllib3
 			if hasattr(ssl, '_create_unverified_context'):
 				ssl._create_default_https_context = ssl._create_unverified_context
-			requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
+			urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 			verify = False
 		else:
 			verify = True
@@ -67,7 +67,7 @@ def soap_login(options):
 
 		conn.service.Login(mo_SessionManager, options["--username"], options["--password"])
 	except requests.exceptions.SSLError as ex:
-		fail_usage("Server side certificate verification failed")
+		fail_usage("Server side certificate verification failed: %s" % ex)
 	except Exception:
 		fail(EC_LOGIN_DENIED)
 
