@@ -10,6 +10,8 @@ import socket
 import textwrap
 import __main__
 
+import itertools
+
 RELEASE_VERSION = "@RELEASE_VERSION@"
 
 __all__ = ['atexit_handler', 'check_input', 'process_input', 'all_opt', 'show_docs',
@@ -351,6 +353,12 @@ all_opt = {
 		"default" : "20",
 		"required" : "0",
 		"order" : 200},
+	"disable_timeout" : {
+		"getopt" : ":",
+		"longopt" : "disable-timeout",
+		"help" : "--disable-timeout=[true/false]     Disable timeout (true/false) (default: true when run from Pacemaker 2.0+)",
+		"required" : "0",
+		"order" : 200},
 	"power_wait" : {
 		"getopt" : ":",
 		"longopt" : "power-wait",
@@ -469,8 +477,8 @@ all_opt = {
 DEPENDENCY_OPT = {
 		"default" : ["help", "debug", "verbose", "verbose_level",
 			 "version", "action", "agent", "power_timeout",
-			 "shell_timeout", "login_timeout", "power_wait",
-			 "retry_on", "delay", "quiet"],
+			 "shell_timeout", "login_timeout", "disable_timeout",
+			 "power_wait", "retry_on", "delay", "quiet"],
 		"passwd" : ["passwd_script"],
 		"sudo" : ["sudo_path"],
 		"secure" : ["identity_file", "ssh_options", "ssh_path", "inet4_only", "inet6_only"],
@@ -492,10 +500,13 @@ class fspawn(pexpect.spawn):
 		self.opt = options
 
 	def log_expect(self, pattern, timeout):
-		result = self.expect(pattern, timeout)
+		result = self.expect(pattern, timeout if timeout != 0 else None)
 		logging.debug("Received: %s", self.before + self.after)
 		return result
 
+	def read_nonblocking(self, size, timeout):
+		return pexpect.spawn.read_nonblocking(self, size=100, timeout=timeout if timeout != 0 else None)
+
 	def send(self, message):
 		logging.debug("Sent: %s", message)
 		return pexpect.spawn.send(self, message)
@@ -508,7 +519,7 @@ def frun(command, timeout=30, withexitstatus=False, events=None,
 	 extra_args=None, logfile=None, cwd=None, env=None, **kwargs):
 	if sys.version_info[0] > 2:
 		kwargs.setdefault('encoding', 'utf-8')
-	return pexpect.run(command, timeout=timeout,
+	return pexpect.run(command, timeout=timeout if timeout != 0 else None,
 			   withexitstatus=withexitstatus, events=events,
 			   extra_args=extra_args, logfile=logfile, cwd=cwd,
 			   env=env, **kwargs)
@@ -769,6 +780,12 @@ def check_input(device_opt, opt, other_conditions = False):
 	if "--password-script" in options:
 		options["--password"] = os.popen(options["--password-script"]).read().rstrip()
 
+	if os.environ.get("PCMK_service") == "pacemaker-fenced" and "--disable-timeout" not in options:
+		options["--disable-timeout"] = "1"
+
+	if options.get("--disable-timeout", "").lower() in ["1", "yes", "on", "true"]:
+		options["--power-timeout"] = options["--shell-timeout"] = options["--login-timeout"] = 0
+
 	return options
 
 ## Obtain a power status from possibly more than one plug
@@ -809,11 +826,15 @@ def async_set_multi_power_fn(connection, options, set_power_fn, get_power_fn, re
 			set_power_fn(connection, options)
 			time.sleep(int(options["--power-wait"]))
 
-		for _ in range(int(options["--power-timeout"])):
+		for _ in itertools.count(1):
 			if get_multi_power_fn(connection, options, get_power_fn) != options["--action"]:
 				time.sleep(1)
 			else:
 				return True
+
+			if int(options["--power-timeout"]) > 0 and _ >= int(options["--power-timeout"]):
+				break
+
 	return False
 
 def sync_set_multi_power_fn(connection, options, sync_set_power_fn, retry_attempts):
@@ -1041,7 +1062,7 @@ def run_command(options, command, timeout=None, env=None, log_command=None):
 
 	thread = threading.Thread(target=process.wait)
 	thread.start()
-	thread.join(timeout)
+	thread.join(timeout if timeout else None)
 	if thread.is_alive():
 		process.kill()
 		fail(EC_TIMED_OUT, stop=(int(options.get("retry", 0)) < 1))
